@@ -10,17 +10,18 @@ use reqwest::Url;
 use tokio::sync::mpsc;
 use tokio_stream::wrappers::ReceiverStream;
 
+use lychee_lib::archive::Archive;
 use lychee_lib::{Client, ErrorKind, Request, Response, Uri};
 use lychee_lib::{InputSource, Result};
 use lychee_lib::{ResponseBody, Status};
 
-use crate::archive::{Archive, Suggestion};
 use crate::formatters::get_response_formatter;
 use crate::formatters::response::ResponseFormatter;
+use crate::formatters::suggestion::Suggestion;
 use crate::options::OutputMode;
 use crate::parse::parse_duration_secs;
 use crate::verbosity::Verbosity;
-use crate::{cache::Cache, stats::ResponseStats, ExitCode};
+use crate::{ExitCode, cache::Cache, stats::ResponseStats};
 
 use super::CommandParams;
 
@@ -48,7 +49,7 @@ where
     let client = params.client;
     let cache = params.cache;
     let cache_exclude_status = params.cfg.cache_exclude_status.into_set();
-    let accept = params.cfg.accept.into_set();
+    let accept = params.cfg.accept.into();
 
     let pb = if params.cfg.no_progress || params.cfg.verbose.log_level() >= log::Level::Info {
         None
@@ -141,7 +142,7 @@ async fn suggest_archived_links(
     let suggestions = Mutex::new(&mut stats.suggestion_map);
 
     futures::stream::iter(failed_urls)
-        .map(|(input, url)| (input, url, archive.get_link(url, timeout)))
+        .map(|(input, url)| (input, url, archive.get_archive_snapshot(url, timeout)))
         .for_each_concurrent(max_concurrency, |(input, url, future)| async {
             if let Ok(Some(suggestion)) = future.await {
                 suggestions
@@ -183,7 +184,7 @@ where
         if let Some(pb) = &bar {
             pb.inc_length(1);
             pb.set_message(request.to_string());
-        };
+        }
         send_req
             .send(Ok(request))
             .await
@@ -268,7 +269,7 @@ async fn check_url(client: &Client, request: Request) -> Response {
     let uri = request.uri.clone();
     let source = request.source.clone();
     client.check(request).await.unwrap_or_else(|e| {
-        log::error!("Error checking URL {}: Cannot parse URL to URI: {}", uri, e);
+        log::error!("Error checking URL {uri}: Cannot parse URL to URI: {e}");
         Response::new(
             uri.clone(),
             Status::Error(ErrorKind::InvalidURI(uri.clone())),
