@@ -4,7 +4,6 @@ use crate::{formatters::get_response_formatter, options, stats::ResponseStats};
 use anyhow::Result;
 use pad::{Alignment, PadStr};
 use std::fmt::{self, Display};
-
 // Maximum padding for each entry in the final statistics output
 const MAX_PADDING: usize = 20;
 
@@ -45,11 +44,12 @@ impl Display for DetailedResponseStats {
         write_stat(f, "\u{1f500} Redirected", stats.redirects, true)?; // 🔀
         write_stat(f, "\u{1f47b} Excluded", stats.excludes, true)?; // 👻
         write_stat(f, "\u{2753} Unknown", stats.unknown, true)?; //❓
-        write_stat(f, "\u{1f6ab} Errors", stats.errors, false)?; // 🚫
+        write_stat(f, "\u{1f6ab} Errors", stats.errors, true)?; // 🚫
+        write_stat(f, "\u{26d4} Unsupported", stats.errors, false)?; // ⛔
 
         let response_formatter = get_response_formatter(&self.mode);
 
-        for (source, responses) in &stats.error_map {
+        for (source, responses) in super::sort_stat_map(&stats.error_map) {
             // Using leading newlines over trailing ones (e.g. `writeln!`)
             // lets us avoid extra newlines without any additional logic.
             write!(f, "\n\nErrors in {source}")?;
@@ -60,12 +60,19 @@ impl Display for DetailedResponseStats {
                     "\n{}",
                     response_formatter.format_detailed_response(response)
                 )?;
+            }
 
-                if let Some(suggestions) = &stats.suggestion_map.get(source) {
-                    writeln!(f, "\nSuggestions in {source}")?;
-                    for suggestion in *suggestions {
-                        writeln!(f, "{suggestion}")?;
-                    }
+            if let Some(suggestions) = stats.suggestion_map.get(source) {
+                // Sort suggestions
+                let mut sorted_suggestions: Vec<_> = suggestions.iter().collect();
+                sorted_suggestions.sort_by(|a, b| {
+                    let (a, b) = (a.to_string().to_lowercase(), b.to_string().to_lowercase());
+                    numeric_sort::cmp(&a, &b)
+                });
+
+                writeln!(f, "\nSuggestions in {source}")?;
+                for suggestion in sorted_suggestions {
+                    writeln!(f, "{suggestion}")?;
                 }
             }
         }
@@ -150,8 +157,10 @@ mod tests {
         assert!(result.contains("❓ Unknown..........0"));
         assert!(result.contains("🚫 Errors...........2"));
         assert!(result.contains("Errors in https://example.com/"));
-        assert!(result
-            .contains("https://github.com/mre/idiomatic-rust-doesnt-exist-man | 404 Not Found"));
+        assert!(
+            result
+                .contains("https://github.com/mre/idiomatic-rust-doesnt-exist-man | 404 Not Found")
+        );
         assert!(result.contains("https://github.com/mre/boom | 500 Internal Server Error"));
     }
 }
