@@ -1,16 +1,22 @@
-use crate::archive::Archive;
 use crate::parse::parse_base;
 use crate::verbosity::Verbosity;
-use anyhow::{anyhow, Context, Error, Result};
+use anyhow::{Context, Error, Result, anyhow};
 use clap::builder::PossibleValuesParser;
-use clap::{arg, builder::TypedValueParser, Parser};
+use clap::{Parser, arg, builder::TypedValueParser};
 use const_format::{concatcp, formatcp};
-use lychee_lib::{
-    Base, BasicAuthSelector, Input, StatusCodeExcluder, StatusCodeSelector, DEFAULT_MAX_REDIRECTS,
-    DEFAULT_MAX_RETRIES, DEFAULT_RETRY_WAIT_TIME_SECS, DEFAULT_TIMEOUT_SECS, DEFAULT_USER_AGENT,
+use http::{
+    HeaderMap,
+    header::{HeaderName, HeaderValue},
 };
+use lychee_lib::{
+    Base, BasicAuthSelector, DEFAULT_MAX_REDIRECTS, DEFAULT_MAX_RETRIES,
+    DEFAULT_RETRY_WAIT_TIME_SECS, DEFAULT_TIMEOUT_SECS, DEFAULT_USER_AGENT, FileExtensions,
+    FileType, Input, StatusCodeExcluder, StatusCodeSelector, archive::Archive,
+};
+use reqwest::tls;
 use secrecy::{ExposeSecret, SecretString};
-use serde::Deserialize;
+use serde::{Deserialize, Deserializer};
+use std::collections::{HashMap, HashSet};
 use std::path::Path;
 use std::{fs, path::PathBuf, str::FromStr, time::Duration};
 use strum::{Display, EnumIter, EnumString, VariantNames};
@@ -45,6 +51,34 @@ const HELP_MSG_CONFIG_FILE: &str = formatcp!(
 const TIMEOUT_STR: &str = concatcp!(DEFAULT_TIMEOUT_SECS);
 const RETRY_WAIT_TIME_STR: &str = concatcp!(DEFAULT_RETRY_WAIT_TIME_SECS);
 
+#[derive(Debug, Deserialize, Default, Clone, Display, EnumIter, EnumString, VariantNames)]
+#[non_exhaustive]
+pub(crate) enum TlsVersion {
+    #[serde(rename = "TLSv1_0")]
+    #[strum(serialize = "TLSv1_0")]
+    V1_0,
+    #[serde(rename = "TLSv1_1")]
+    #[strum(serialize = "TLSv1_1")]
+    V1_1,
+    #[serde(rename = "TLSv1_2")]
+    #[strum(serialize = "TLSv1_2")]
+    #[default]
+    V1_2,
+    #[serde(rename = "TLSv1_3")]
+    #[strum(serialize = "TLSv1_3")]
+    V1_3,
+}
+impl From<TlsVersion> for tls::Version {
+    fn from(ver: TlsVersion) -> Self {
+        match ver {
+            TlsVersion::V1_0 => tls::Version::TLS_1_0,
+            TlsVersion::V1_1 => tls::Version::TLS_1_1,
+            TlsVersion::V1_2 => tls::Version::TLS_1_2,
+            TlsVersion::V1_3 => tls::Version::TLS_1_3,
+        }
+    }
+}
+
 /// The format to use for the final status report
 #[derive(Debug, Deserialize, Default, Clone, Display, EnumIter, VariantNames, PartialEq)]
 #[non_exhaustive]
@@ -171,6 +205,103 @@ macro_rules! fold_in {
     };
 }
 
+/// Parse a single header into a [`HeaderName`] and [`HeaderValue`]
+///
+/// Headers are expected to be in format `Header-Name: Header-Value`.
+/// The header name and value are trimmed of whitespace.
+///
+/// If the header contains multiple colons, the part after the first colon is
+/// considered the value.
+///
+/// # Errors
+///
+/// This fails if the header does not contain exactly one `:` character or
+/// if the header name contains non-ASCII characters.
+fn parse_single_header(header: &str) -> Result<(HeaderName, HeaderValue)> {
+    let parts: Vec<&str> = header.splitn(2, ':').collect();
+    match parts.as_slice() {
+        [name, value] => {
+            let name = HeaderName::from_str(name.trim())
+                .map_err(|e| anyhow!("Unable to convert header name '{}': {}", name.trim(), e))?;
+            let value = HeaderValue::from_str(value.trim()).map_err(|e| {
+                anyhow!("Unable to read value of header with name '{}': {}", name, e)
+            })?;
+            Ok((name, value))
+        }
+        _ => Err(anyhow!(
+            "Invalid header format. Expected colon-separated string in the format 'HeaderName: HeaderValue'"
+        )),
+    }
+}
+
+/// Parses a single HTTP header into a tuple of (String, String)
+///
+/// This does NOT merge multiple headers into one.
+#[derive(Clone, Debug)]
+struct HeaderParser;
+
+impl TypedValueParser for HeaderParser {
+    type Value = (String, String);
+
+    fn parse_ref(
+        &self,
+        _cmd: &clap::Command,
+        _arg: Option<&clap::Arg>,
+        value: &std::ffi::OsStr,
+    ) -> Result<Self::Value, clap::Error> {
+        let header_str = value.to_str().ok_or_else(|| {
+            clap::Error::raw(
+                clap::error::ErrorKind::InvalidValue,
+                "Header value contains invalid UTF-8",
+            )
+        })?;
+
+        match parse_single_header(header_str) {
+            Ok((name, value)) => {
+                let Ok(value) = value.to_str() else {
+                    return Err(clap::Error::raw(
+                        clap::error::ErrorKind::InvalidValue,
+                        "Header value contains invalid UTF-8",
+                    ));
+                };
+
+                Ok((name.to_string(), value.to_string()))
+            }
+            Err(e) => Err(clap::Error::raw(
+                clap::error::ErrorKind::InvalidValue,
+                e.to_string(),
+            )),
+        }
+    }
+}
+
+impl clap::builder::ValueParserFactory for HeaderParser {
+    type Parser = HeaderParser;
+    fn value_parser() -> Self::Parser {
+        HeaderParser
+    }
+}
+
+/// Extension trait for converting a Vec of header pairs to a `HeaderMap`
+pub(crate) trait HeaderMapExt {
+    /// Convert a collection of header key-value pairs to a `HeaderMap`
+    fn from_header_pairs(headers: &[(String, String)]) -> Result<HeaderMap, Error>;
+}
+
+impl HeaderMapExt for HeaderMap {
+    fn from_header_pairs(headers: &[(String, String)]) -> Result<HeaderMap, Error> {
+        let mut header_map = HeaderMap::new();
+        for (name, value) in headers {
+            let header_name = HeaderName::from_bytes(name.as_bytes())
+                .map_err(|e| anyhow!("Invalid header name '{}': {}", name, e))?;
+            let header_value = HeaderValue::from_str(value)
+                .map_err(|e| anyhow!("Invalid header value '{}': {}", value, e))?;
+            header_map.insert(header_name, header_value);
+        }
+        Ok(header_map)
+    }
+}
+
 /// A fast, async link checker
 ///
 /// Finds broken URLs and mail addresses inside Markdown, HTML,
@@ -199,20 +330,24 @@ impl LycheeOptions {
     // This depends on the config, which is why a method is required (we could
     // accept a `Vec<Input>` in `LycheeOptions` and do the conversion there, but
     // we wouldn't get access to `glob_ignore_case`.
-    pub(crate) fn inputs(&self) -> Result<Vec<Input>> {
-        let excluded = if self.config.exclude_path.is_empty() {
-            None
-        } else {
-            Some(self.config.exclude_path.clone())
-        };
+    pub(crate) fn inputs(&self) -> Result<HashSet<Input>> {
         self.raw_inputs
             .iter()
-            .map(|s| Input::new(s, None, self.config.glob_ignore_case, excluded.clone()))
+            .map(|raw_input| Input::new(raw_input, None, self.config.glob_ignore_case))
             .collect::<Result<_, _>>()
             .context("Cannot parse inputs from arguments")
     }
 }
 
+// Custom deserializer function for the header field
+fn deserialize_headers<'de, D>(deserializer: D) -> Result<Vec<(String, String)>, D::Error>
+where
+    D: Deserializer<'de>,
+{
+    let map = HashMap::<String, String>::deserialize(deserializer)?;
+    Ok(map.into_iter().collect())
+}
+
 /// The main configuration for lychee
 #[allow(clippy::struct_excessive_bools)]
 #[derive(Parser, Debug, Deserialize, Clone, Default)]
@@ -228,6 +363,25 @@ pub(crate) struct Config {
     #[serde(default)]
     pub(crate) no_progress: bool,
 
+    /// A list of file extensions. Files not matching the specified extensions are skipped.
+    ///
+    /// E.g. a user can specify `--extensions html,htm,php,asp,aspx,jsp,cgi`
+    /// to check for links in files with these extensions.
+    ///
+    /// This is useful when the default extensions are not enough and you don't
+    /// want to provide a long list of inputs (e.g. file1.html, file2.md, etc.)
+    #[arg(
+        long,
+        default_value_t = FileExtensions::default(),
+        long_help = "Test the specified file extensions for URIs when checking files locally.
+
+Multiple extensions can be separated by commas. Note that if you want to check filetypes,
+which have multiple extensions, e.g. HTML files with both .html and .htm extensions, you need to
+specify both extensions explicitly."
+    )]
+    #[serde(default = "FileExtensions::default")]
+    pub(crate) extensions: FileExtensions,
+
     #[arg(help = HELP_MSG_CACHE)]
     #[arg(long)]
     #[serde(default)]
@@ -249,16 +403,18 @@ pub(crate) struct Config {
         default_value_t,
         long_help = "A list of status codes that will be ignored from the cache
 
-The following accept range syntax is supported: [start]..[=]end|code. Some valid
+The following exclude range syntax is supported: [start]..[[=]end]|code. Some valid
 examples are:
 
-- 429
-- 500..=599
-- 500..
+- 429 (excludes the 429 status code only)
+- 500.. (excludes any status code >= 500)
+- ..100 (excludes any status code < 100)
+- 500..=599 (excludes any status code from 500 to 599 inclusive)
+- 500..600 (excludes any status code from 500 to 600 excluding 600, same as 500..=599)
 
 Use \"lychee --cache-exclude-status '429, 500..502' <inputs>...\" to provide a comma- separated
-list of excluded status codes. This example will not cache results with a status code of 429, 500,
-501 and 502."
+list of excluded status codes. This example will not cache results with a status code of 429, 500
+and 501."
     )]
     #[serde(default = "cache_exclude_selector")]
     pub(crate) cache_exclude_status: StatusCodeExcluder,
@@ -297,6 +453,11 @@ list of excluded status codes. This example will not cache results with a status
     #[serde(default = "max_retries")]
     pub(crate) max_retries: u64,
 
+    /// Minimum accepted TLS Version
+    #[arg(long, value_parser = PossibleValuesParser::new(TlsVersion::VARIANTS).map(|s| s.parse::<TlsVersion>().unwrap()))]
+    #[serde(default)]
+    pub(crate) min_tls: Option<TlsVersion>,
+
     /// Maximum number of concurrent network requests
     #[arg(long, default_value = &MAX_CONCURRENCY_STR)]
     #[serde(default = "max_concurrency")]
@@ -335,7 +496,8 @@ list of excluded status codes. This example will not cache results with a status
     #[serde(default)]
     pub(crate) include: Vec<String>,
 
-    /// Exclude URLs and mail addresses from checking (supports regex)
+    /// Exclude URLs and mail addresses from checking.
+    /// The values are treated as regular expressions.
     #[arg(long)]
     #[serde(default)]
     pub(crate) exclude: Vec<String>,
@@ -345,10 +507,11 @@ list of excluded status codes. This example will not cache results with a status
     #[serde(default)]
     pub(crate) exclude_file: Vec<String>,
 
-    /// Exclude file path from getting checked.
+    /// Exclude paths from getting checked.
+    /// The values are treated as regular expressions.
     #[arg(long)]
     #[serde(default)]
-    pub(crate) exclude_path: Vec<PathBuf>,
+    pub(crate) exclude_path: Vec<String>,
 
     /// Exclude all private IPs from checking.
     /// Equivalent to `--exclude-private --exclude-link-local --exclude-loopback`
@@ -371,12 +534,6 @@ list of excluded status codes. This example will not cache results with a status
     #[serde(default)]
     pub(crate) exclude_loopback: bool,
 
-    /// Exclude all mail addresses from checking
-    /// (deprecated; excluded by default)
-    #[arg(long)]
-    #[serde(default)]
-    pub(crate) exclude_mail: bool,
-
     /// Also check email addresses
     #[arg(long)]
     #[serde(default)]
@@ -387,23 +544,72 @@ list of excluded status codes. This example will not cache results with a status
     #[arg(long)]
     pub(crate) remap: Vec<String>,
 
-    /// Automatically append file extensions to `file://` URIs as needed
+    /// Automatically append file extensions to `file://` URIs for non-existing paths
     #[serde(default)]
     #[arg(
         long,
         value_delimiter = ',',
-        long_help = "Test the specified file extensions for URIs when checking files locally.
-Multiple extensions can be separated by commas. Extensions will be checked in
-order of appearance.
+        long_help = "When checking locally, attempts to locate missing files by trying the given
+fallback extensions. Multiple extensions can be separated by commas. Extensions
+will be checked in order of appearance.
 
-Example: --fallback-extensions html,htm,php,asp,aspx,jsp,cgi"
+Example: --fallback-extensions html,htm,php,asp,aspx,jsp,cgi
+
+Note: This option only takes effect on `file://` URIs which do not exist."
     )]
     pub(crate) fallback_extensions: Vec<String>,
 
-    /// Custom request header
-    #[arg(long)]
+    /// Resolve local directory links to specified index files within the directory
     #[serde(default)]
-    pub(crate) header: Vec<String>,
+    #[arg(
+        long,
+        value_delimiter = ',',
+        long_help = "When checking locally, resolves directory links to a separate index file.
+The argument is a comma-separated list of index file names to search for. Index
+names are relative to the link's directory and attempted in the order given.
+
+If `--index-files` is specified, then at least one index file must exist in
+order for a directory link to be considered valid. Additionally, the special
+name `.` can be used in the list to refer to the directory itself.
+
+If unspecified (the default behavior), index files are disabled and directory
+links are considered valid as long as the directory exists on disk.
+
+Example 1: `--index-files index.html,readme.md` looks for index.html or readme.md
+           and requires that at least one exists.
+
+Example 2: `--index-files index.html,.` will use index.html if it exists, but
+           still accept the directory link regardless.
+
+Example 3: `--index-files ''` will reject all directory links because there are
+           no valid index files. This will require every link to explicitly name
+           a file.
+
+Note: This option only takes effect on `file://` URIs which exist and point to a directory."
+    )]
+    pub(crate) index_files: Option<Vec<String>>,
+
+    /// Set custom header for requests
+    #[arg(
+        short = 'H',
+        long = "header",
+        // Note: We use a `Vec<(String, String)>` for headers, which is
+        // unfortunate. The reason is that `clap::ArgAction::Append` collects
+        // multiple values, and `clap` cannot automatically convert these tuples
+        // into a `HashMap<String, String>`.
+        action = clap::ArgAction::Append,
+        value_parser = HeaderParser,
+        value_name = "HEADER:VALUE",
+        long_help = "Set custom header for requests
+
+Some websites require custom headers to be passed in order to return valid responses.
+You can specify custom headers in the format 'Name: Value'. For example, 'Accept: text/html'.
+This is the same format that other tools like curl or wget use.
+Multiple headers can be specified by using the flag multiple times."
+    )]
+    #[serde(default)]
+    #[serde(deserialize_with = "deserialize_headers")]
+    pub header: Vec<(String, String)>,
 
     /// A List of accepted status codes for valid links
     #[arg(
@@ -412,14 +618,14 @@ Example: --fallback-extensions html,htm,php,asp,aspx,jsp,cgi"
         default_value_t,
         long_help = "A List of accepted status codes for valid links
 
-The following accept range syntax is supported: [start]..[=]end|code. Some valid
+The following accept range syntax is supported: [start]..[[=]end]|code. Some valid
 examples are:
 
-- 200..=204
-- 200..204
-- ..=204
-- ..204
-- 200
+- 200 (accepts the 200 status code only)
+- ..204 (accepts any status code < 204)
+- ..=204 (accepts any status code <= 204)
+- 200..=204 (accepts any status code from 200 to 204 inclusive)
+- 200..205 (accepts any status code from 200 to 205 excluding 205, same as 200..=204)
 
 Use \"lychee --accept '200..=204, 429, 500' <inputs>...\" to provide a comma-
 separated list of accepted status codes. This example will accept 200, 201,
@@ -449,11 +655,16 @@ separated list of accepted status codes. This example will accept 200, 201,
     #[serde(default = "method")]
     pub(crate) method: String,
 
-    /// Base URL or website root directory to check relative URLs
-    /// e.g. <https://example.com> or `/path/to/public`
+    /// Deprecated; use `--base-url` instead
+    #[arg(long, value_parser = parse_base)]
+    #[serde(skip)]
+    pub(crate) base: Option<Base>,
+
+    /// Base URL used to resolve relative URLs during link checking
+    /// Example: <https://example.com>
     #[arg(short, long, value_parser= parse_base)]
     #[serde(default)]
-    pub(crate) base: Option<Base>,
+    pub(crate) base_url: Option<Base>,
 
     /// Root path to use when checking absolute local links,
     /// must be an absolute path
@@ -523,9 +734,29 @@ separated list of accepted status codes. This example will accept 200, 201,
     #[arg(long)]
     #[serde(default)]
     pub(crate) cookie_jar: Option<PathBuf>,
+
+    #[allow(clippy::doc_markdown)]
+    /// Check WikiLinks in Markdown files
+    #[arg(long)]
+    #[serde(default)]
+    pub(crate) include_wikilinks: bool,
 }
 
 impl Config {
+    /// Special handling for merging headers
+    ///
+    /// Overwrites existing headers in `self` with the values from `other`.
+    fn merge_headers(&mut self, other: &[(String, String)]) {
+        let self_map = self.header.iter().cloned().collect::<HashMap<_, _>>();
+        let other_map = other.iter().cloned().collect::<HashMap<_, _>>();
+
+        // Merge the two maps, with `other` taking precedence
+        let merged_map: HashMap<_, _> = self_map.into_iter().chain(other_map).collect();
+
+        // Convert the merged map back to a Vec of tuples
+        self.header = merged_map.into_iter().collect();
+    }
+
     /// Load configuration from a file
     pub(crate) fn load_from_file(path: &Path) -> Result<Config> {
         // Read configuration file
@@ -535,52 +766,58 @@ impl Config {
 
     /// Merge the configuration from TOML into the CLI configuration
     pub(crate) fn merge(&mut self, toml: Config) {
+        // Special handling for headers before fold_in!
+        self.merge_headers(&toml.header);
+
         fold_in! {
             // Destination and source configs
             self, toml;
 
             // Keys with defaults to assign
-            verbose: Verbosity::default();
-            cache: false;
-            no_progress: false;
-            max_redirects: DEFAULT_MAX_REDIRECTS;
-            max_retries: DEFAULT_MAX_RETRIES;
-            max_concurrency: DEFAULT_MAX_CONCURRENCY;
-            max_cache_age: humantime::parse_duration(DEFAULT_MAX_CACHE_AGE).unwrap();
+            accept: StatusCodeSelector::default();
+            base_url: None;
+            basic_auth: None;
             cache_exclude_status: StatusCodeExcluder::default();
-            threads: None;
-            user_agent: DEFAULT_USER_AGENT;
-            insecure: false;
-            scheme: Vec::<String>::new();
-            include: Vec::<String>::new();
-            exclude: Vec::<String>::new();
-            exclude_file: Vec::<String>::new(); // deprecated
-            exclude_path: Vec::<PathBuf>::new();
+            cache: false;
+            cookie_jar: None;
             exclude_all_private: false;
-            exclude_private: false;
+            exclude_file: Vec::<String>::new(); // deprecated
             exclude_link_local: false;
             exclude_loopback: false;
-            exclude_mail: false;
-            format: StatsFormat::default();
-            remap: Vec::<String>::new();
+            exclude_path: Vec::<String>::new();
+            exclude_private: false;
+            exclude: Vec::<String>::new();
+            extensions: FileType::default_extensions();
             fallback_extensions: Vec::<String>::new();
-            header: Vec::<String>::new();
-            timeout: DEFAULT_TIMEOUT_SECS;
-            retry_wait_time: DEFAULT_RETRY_WAIT_TIME_SECS;
-            method: DEFAULT_METHOD;
-            base: None;
-            basic_auth: None;
-            skip_missing: false;
-            include_verbatim: false;
-            include_mail: false;
+            format: StatsFormat::default();
             glob_ignore_case: false;
-            output: None;
-            require_https: false;
-            cookie_jar: None;
+            header: Vec::<(String, String)>::new();
             include_fragments: false;
-            accept: StatusCodeSelector::default();
+            include_mail: false;
+            include_verbatim: false;
+            include_wikilinks: false;
+            include: Vec::<String>::new();
+            insecure: false;
+            max_cache_age: humantime::parse_duration(DEFAULT_MAX_CACHE_AGE).unwrap();
+            max_concurrency: DEFAULT_MAX_CONCURRENCY;
+            max_redirects: DEFAULT_MAX_REDIRECTS;
+            max_retries: DEFAULT_MAX_RETRIES;
+            method: DEFAULT_METHOD;
+            no_progress: false;
+            output: None;
+            remap: Vec::<String>::new();
+            require_https: false;
+            retry_wait_time: DEFAULT_RETRY_WAIT_TIME_SECS;
+            scheme: Vec::<String>::new();
+            skip_missing: false;
+            threads: None;
+            timeout: DEFAULT_TIMEOUT_SECS;
+            user_agent: DEFAULT_USER_AGENT;
+            verbose: Verbosity::default();
         }
 
+        // If the config file has a value for the GitHub token, but the CLI
+        // doesn't, use the token from the config file.
         if self
             .github_token
             .as_ref()
@@ -599,6 +836,8 @@ impl Config {
 
 #[cfg(test)]
 mod tests {
+    use std::collections::HashMap;
+
     use super::*;
 
     #[test]
@@ -628,4 +867,105 @@ mod tests {
         );
         assert_eq!(cli.cache_exclude_status, StatusCodeExcluder::new());
     }
+
+    #[test]
+    fn test_parse_custom_headers() {
+        assert_eq!(
+            parse_single_header("accept:text/html").unwrap(),
+            (
+                HeaderName::from_static("accept"),
+                HeaderValue::from_static("text/html")
+            )
+        );
+    }
+
+    #[test]
+    fn test_parse_custom_header_multiple_colons() {
+        assert_eq!(
+            parse_single_header("key:x-test:check=this").unwrap(),
+            (
+                HeaderName::from_static("key"),
+                HeaderValue::from_static("x-test:check=this")
+            )
+        );
+    }
+
+    #[test]
+    fn test_parse_custom_headers_with_equals() {
+        assert_eq!(
+            parse_single_header("key:x-test=check=this").unwrap(),
+            (
+                HeaderName::from_static("key"),
+                HeaderValue::from_static("x-test=check=this")
+            )
+        );
+    }
+
+    #[test]
+    /// We should not reveal potentially sensitive data contained in the headers.
+    /// See: [#1297](https://github.com/lycheeverse/lychee/issues/1297)
+    fn test_does_not_echo_sensitive_data() {
+        let error = parse_single_header("My-Header💣: secret")
+            .expect_err("Should not allow unicode as key");
+        assert!(!error.to_string().contains("secret"));
+
+        let error = parse_single_header("secret").expect_err("Should fail when no `:` given");
+        assert!(!error.to_string().contains("secret"));
+    }
+
+    #[test]
+    fn test_header_parsing_and_merging() {
+        // Simulate commandline arguments with multiple headers
+        let args = vec![
+            "lychee",
+            "--header",
+            "Accept: text/html",
+            "--header",
+            "X-Test: check=this",
+            "input.md",
+        ];
+
+        // Parse the arguments
+        let opts = crate::LycheeOptions::parse_from(args);
+
+        // Check that the headers were collected correctly
+        let headers = &opts.config.header;
+        assert_eq!(headers.len(), 2);
+
+        // Convert to HashMap for easier testing
+        let header_map: HashMap<String, String> = headers.iter().cloned().collect();
+        assert_eq!(header_map["accept"], "text/html");
+        assert_eq!(header_map["x-test"], "check=this");
+    }
+
+    #[test]
+    fn test_merge_headers_with_config() {
+        let toml = Config {
+            header: vec![
+                ("Accept".to_string(), "text/html".to_string()),
+                ("X-Test".to_string(), "check=this".to_string()),
+            ],
+            ..Default::default()
+        };
+
+        // Set X-Test and see if it gets overwritten
+        let mut cli = Config {
+            header: vec![("X-Test".to_string(), "check=that".to_string())],
+            ..Default::default()
+        };
+        cli.merge(toml);
+
+        assert_eq!(cli.header.len(), 2);
+
+        // Sort vector before assert
+        cli.header.sort();
+
+        assert_eq!(
+            cli.header,
+            vec![
+                ("Accept".to_string(), "text/html".to_string()),
+                ("X-Test".to_string(), "check=this".to_string()),
+            ]
+        );
+    }
 }
