@@ -4,18 +4,18 @@ mod cli {
         collections::{HashMap, HashSet},
         error::Error,
         fs::{self, File},
-        io::Write,
+        io::{BufRead, Write},
         path::{Path, PathBuf},
         time::Duration,
     };
 
     use anyhow::anyhow;
-    use assert_cmd::Command;
+    use assert_cmd::{Command, assert::Assert};
     use assert_json_diff::assert_json_include;
-    use http::StatusCode;
+    use http::{Method, StatusCode};
     use lychee_lib::{InputSource, ResponseBody};
     use predicates::{
-        prelude::{predicate, PredicateBooleanExt},
+        prelude::{PredicateBooleanExt, predicate},
         str::{contains, is_empty},
     };
     use pretty_assertions::assert_eq;
@@ -24,7 +24,10 @@ mod cli {
     use serde_json::Value;
     use tempfile::NamedTempFile;
     use uuid::Uuid;
-    use wiremock::{matchers::basic_auth, Mock, ResponseTemplate};
+    use wiremock::{
+        Mock, ResponseTemplate,
+        matchers::{basic_auth, method},
+    };
 
     type Result<T> = std::result::Result<T, Box<dyn Error>>;
 
@@ -33,7 +36,7 @@ mod cli {
     // constant.
     const LYCHEE_CACHE_FILE: &str = ".lycheecache";
 
-    /// Helper macro to create a mock server which returns a custom status code.
+    /// Create a mock server which returns a custom status code.
     macro_rules! mock_server {
         ($status:expr $(, $func:tt ($($arg:expr),*))*) => {{
             let mock_server = wiremock::MockServer::start().await;
@@ -44,7 +47,7 @@ mod cli {
         }};
     }
 
-    /// Helper macro to create a mock server which returns a 200 OK and a custom response body.
+    /// Create a mock server which returns a 200 OK and a custom response body.
     macro_rules! mock_response {
         ($body:expr) => {{
             let mock_server = wiremock::MockServer::start().await;
@@ -62,7 +65,7 @@ mod cli {
         Command::cargo_bin(env!("CARGO_PKG_NAME")).expect("Couldn't get cargo package name")
     }
 
-    /// Helper function to get the root path of the project.
+    /// Get the root path of the project.
     fn root_path() -> PathBuf {
         Path::new(env!("CARGO_MANIFEST_DIR"))
             .parent()
@@ -70,11 +73,58 @@ mod cli {
             .to_path_buf()
     }
 
-    /// Helper function to get the path to the fixtures directory.
+    /// Get the path to the fixtures directory.
     fn fixtures_path() -> PathBuf {
         root_path().join("fixtures")
     }
 
+    /// Convert a relative path to an absolute path string
+    /// starting from a base directory.
+    fn path_str(base: &Path, relative_path: &str) -> String {
+        base.join(relative_path).to_string_lossy().to_string()
+    }
+
+    /// Assert actual output lines equals to expected lines.
+    /// Order of the lines is ignored.
+    fn assert_lines_eq<S: AsRef<str> + Ord>(result: Assert, mut expected_lines: Vec<S>) {
+        let output = result.get_output().stdout.clone();
+        let mut actual_lines: Vec<String> = output
+            .lines()
+            .map(|line| line.unwrap().to_string())
+            .collect();
+
+        actual_lines.sort();
+        expected_lines.sort();
+
+        let expected_lines: Vec<String> = expected_lines
+            .into_iter()
+            .map(|l| l.as_ref().to_owned())
+            .collect();
+
+        assert_eq!(actual_lines, expected_lines);
+    }
+
+    /// Test the output of the JSON format.
+    macro_rules! test_json_output {
+        ($test_file:expr, $expected:expr $(, $arg:expr)*) => {{
+            let mut cmd = main_command();
+            let test_path = fixtures_path().join($test_file);
+            let outfile = format!("{}.json", uuid::Uuid::new_v4());
+
+            let result = cmd$(.arg($arg))*.arg("--output").arg(&outfile).arg("--format").arg("json").arg(test_path).assert();
+
+            let output = std::fs::read_to_string(&outfile)?;
+            std::fs::remove_file(outfile)?;
+
+            let actual: Value = serde_json::from_str(&output)?;
+            let expected: Value = serde_json::to_value(&$expected)?;
+
+            result.success();
+            assert_json_include!(actual: actual, expected: expected);
+            Ok(())
+        }};
+    }
+
     #[derive(Default, Serialize)]
     struct MockResponseStats {
         detailed_stats: bool,
@@ -93,26 +143,6 @@ mod cli {
         excluded_map: HashMap<InputSource, HashSet<ResponseBody>>,
     }
 
-    /// Helper macro to test the output of the JSON format.
-    macro_rules! test_json_output {
-        ($test_file:expr, $expected:expr $(, $arg:expr)*) => {{
-            let mut cmd = main_command();
-            let test_path = fixtures_path().join($test_file);
-            let outfile = format!("{}.json", uuid::Uuid::new_v4());
-
-            cmd$(.arg($arg))*.arg("--output").arg(&outfile).arg("--format").arg("json").arg(test_path).assert().success();
-
-            let output = std::fs::read_to_string(&outfile)?;
-            std::fs::remove_file(outfile)?;
-
-            let actual: Value = serde_json::from_str(&output)?;
-            let expected: Value = serde_json::to_value(&$expected)?;
-
-            assert_json_include!(actual: actual, expected: expected);
-            Ok(())
-        }};
-    }
-
     /// Test that the default report output format (compact) and mode (color)
     /// prints the failed URLs as well as their status codes on error. Make
     /// sure that the status code only occurs once.
@@ -276,6 +306,18 @@ mod cli {
         )
     }
 
+    #[test]
+    fn test_local_directories() -> Result<()> {
+        test_json_output!(
+            "TEST_LOCAL_DIRECTORIES.md",
+            MockResponseStats {
+                total: 4,
+                successful: 4,
+                ..MockResponseStats::default()
+            }
+        )
+    }
+
     #[test]
     fn test_email() -> Result<()> {
         test_json_output!(
@@ -383,7 +425,7 @@ mod cli {
         let dir = fixtures_path().join("resolve_paths");
 
         cmd.arg("--offline")
-            .arg("--base")
+            .arg("--base-url")
             .arg(&dir)
             .arg(dir.join("index.html"))
             .env_clear()
@@ -419,7 +461,7 @@ mod cli {
         cmd.arg("--offline")
             .arg("--root-dir")
             .arg("/resolve_paths")
-            .arg("--base")
+            .arg("--base-url")
             .arg(&dir)
             .arg(dir.join("resolve_paths").join("index.html"))
             .env_clear()
@@ -562,7 +604,7 @@ mod cli {
             .failure()
             .code(2)
             .stdout(contains(
-                "[404] https://github.com/mre/idiomatic-rust-doesnt-exist-man | Network error: Not Found"
+                r#"[404] https://github.com/mre/idiomatic-rust-doesnt-exist-man | Rejected status code (this depends on your "accept" configuration): Not Found"#
             ))
             .stderr(contains(
                 "There were issues with GitHub URLs. You could try setting a GitHub token and running lychee again.",
@@ -815,6 +857,19 @@ mod cli {
         Ok(())
     }
 
+    #[test]
+    fn test_invalid_default_config() -> Result<()> {
+        let test_path = fixtures_path().join("configs");
+        let mut cmd = main_command();
+        cmd.current_dir(test_path)
+            .arg(".")
+            .assert()
+            .failure()
+            .stderr(contains("Cannot load default configuration file"));
+
+        Ok(())
+    }
+
     #[tokio::test]
     async fn test_include_mail_config() -> Result<()> {
         let test_mail_address = "mailto:hello-test@testingabc.io";
@@ -880,8 +935,7 @@ mod cli {
             .failure()
             .stderr(predicate::str::contains("Cannot load configuration file"))
             .stderr(predicate::str::contains("Failed to parse"))
-            .stderr(predicate::str::contains("TOML parse error"))
-            .stderr(predicate::str::contains("expected newline"));
+            .stderr(predicate::str::contains("TOML parse error"));
     }
 
     #[tokio::test]
@@ -985,7 +1039,7 @@ mod cli {
 
         // Ensure clean state
         if cache_file.exists() {
-            println!("Removing cache file before test: {:?}", cache_file);
+            println!("Removing cache file before test: {cache_file:?}");
             fs::remove_file(&cache_file)?;
             tokio::time::sleep(Duration::from_millis(500)).await;
         }
@@ -1028,7 +1082,7 @@ mod cli {
 
         // Check cache contents
         let data = fs::read_to_string(&cache_file)?;
-        println!("Cache file contents: {}", data);
+        println!("Cache file contents: {data}");
 
         assert!(
             data.contains(&format!("{}/,200", mock_server_ok.uri())),
@@ -1105,7 +1159,7 @@ mod cli {
                 mock_server_no_content.uri()
             )))
             .stderr(contains(format!(
-                "[429] {}/ | Network error: Too Many Requests\n",
+                "[429] {}/ | Rejected status code (this depends on your \"accept\" configuration): Too Many Requests\n",
                 mock_server_too_many_requests.uri()
             )));
 
@@ -1163,11 +1217,11 @@ mod cli {
             .failure()
             .code(2)
             .stdout(contains(format!(
-                "[418] {}/ | Network error: I\'m a teapot",
+                r#"[418] {}/ | Rejected status code (this depends on your "accept" configuration): I'm a teapot"#,
                 mock_server_teapot.uri()
             )))
             .stdout(contains(format!(
-                "[500] {}/ | Network error: Internal Server Error",
+                r#"[500] {}/ | Rejected status code (this depends on your "accept" configuration): Internal Server Error"#,
                 mock_server_server_error.uri()
             )));
 
@@ -1200,6 +1254,26 @@ mod cli {
         Ok(())
     }
 
+    #[tokio::test]
+    async fn test_accept_overrides_defaults_not_additive() -> Result<()> {
+        let mock_server_200 = mock_server!(StatusCode::OK);
+
+        let mut cmd = main_command();
+        cmd.arg("--accept")
+            .arg("404") // ONLY accept 404 - should reject 200 as we overwrite the default
+            .arg("-")
+            .write_stdin(mock_server_200.uri())
+            .assert()
+            .failure()
+            .code(2)
+            .stdout(contains(format!(
+                r#"[200] {}/ | Rejected status code (this depends on your "accept" configuration): OK"#,
+                mock_server_200.uri()
+            )));
+
+        Ok(())
+    }
+
     #[tokio::test]
     async fn test_skip_cache_unsupported() -> Result<()> {
         let base_path = fixtures_path().join("cache");
@@ -1526,25 +1600,30 @@ mod cli {
     }
 
     #[test]
-    fn test_excluded_paths() -> Result<()> {
+    fn test_excluded_paths_regex() -> Result<()> {
         let test_path = fixtures_path().join("exclude-path");
-
-        let excluded_path1 = test_path.join("dir1");
-        let excluded_path2 = test_path.join("dir2").join("subdir");
+        let excluded_path_1 = "\\/excluded?\\/"; // exclude paths containing a directory "exclude" and "excluded"
+        let excluded_path_2 = "(\\.mdx|\\.txt)$"; // exclude .mdx and .txt files
         let mut cmd = main_command();
 
-        cmd.arg("--exclude-path")
-            .arg(&excluded_path1)
+        let result = cmd
             .arg("--exclude-path")
-            .arg(&excluded_path2)
+            .arg(excluded_path_1)
+            .arg("--exclude-path")
+            .arg(excluded_path_2)
+            .arg("--dump")
             .arg("--")
             .arg(&test_path)
             .assert()
-            .success()
-            // Links in excluded files are not taken into account in the total
-            // number of links.
-            .stdout(contains("1 Total"))
-            .stdout(contains("1 OK"));
+            .success();
+
+        assert_lines_eq(
+            result,
+            vec![
+                "https://test.md/to-be-included-outer",
+                "https://test.md/to-be-included-inner",
+            ],
+        );
 
         Ok(())
     }
@@ -1589,7 +1668,7 @@ mod cli {
     #[test]
     fn test_prevent_too_many_redirects() -> Result<()> {
         let mut cmd = main_command();
-        let url = "https://httpstat.us/308";
+        let url = "https://http.codes/308";
 
         cmd.write_stdin(url)
             .arg("--max-redirects")
@@ -1640,8 +1719,14 @@ mod cli {
         let password = "password123";
 
         let mock_server = wiremock::MockServer::start().await;
-        Mock::given(basic_auth(username, password))
-            .respond_with(ResponseTemplate::new(200))
+
+        Mock::given(method("GET"))
+            .and(basic_auth(username, password))
+            .respond_with(ResponseTemplate::new(200)) // Authenticated requests are accepted
+            .mount(&mock_server)
+            .await;
+        Mock::given(method("GET"))
+            .respond_with(|_: &_| panic!("Received unauthenticated request"))
             .mount(&mock_server)
             .await;
 
@@ -1657,6 +1742,16 @@ mod cli {
             .stdout(contains("1 Total"))
             .stdout(contains("1 OK"));
 
+        // Websites as direct arguments must also use authentication
+        main_command()
+            .arg(mock_server.uri())
+            .arg("--verbose")
+            .arg("--basic-auth")
+            .arg(format!("{} {username}:{password}", mock_server.uri()))
+            .assert()
+            .success()
+            .stdout(contains("0 Total")); // Mock server returns no body, so there are no URLs to check
+
         Ok(())
     }
 
@@ -1719,6 +1814,38 @@ mod cli {
         assert!(all_cookies.iter().all(|c| c.domain() == Some("google.com")));
         Ok(())
     }
+
+    #[test]
+    fn test_dump_inputs_does_not_include_duplicates() -> Result<()> {
+        let pattern = fixtures_path().join("dump_inputs/markdown.md");
+
+        let mut cmd = main_command();
+        cmd.arg("--dump-inputs")
+            .arg(&pattern)
+            .arg(&pattern)
+            .assert()
+            .success()
+            .stdout(contains("fixtures/dump_inputs/markdown.md").count(1));
+
+        Ok(())
+    }
+
+    #[test]
+    fn test_dump_inputs_glob_does_not_include_duplicates() -> Result<()> {
+        let pattern1 = fixtures_path().join("**/markdown.*");
+        let pattern2 = fixtures_path().join("**/*.md");
+
+        let mut cmd = main_command();
+        cmd.arg("--dump-inputs")
+            .arg(pattern1)
+            .arg(pattern2)
+            .assert()
+            .success()
+            .stdout(contains("fixtures/dump_inputs/markdown.md").count(1));
+
+        Ok(())
+    }
+
     #[test]
     fn test_dump_inputs_glob_md() -> Result<()> {
         let pattern = fixtures_path().join("**/*.md");
@@ -1747,7 +1874,6 @@ mod cli {
             .stdout(contains("fixtures/dump_inputs/subfolder/file2.md"))
             .stdout(contains("fixtures/dump_inputs/subfolder"))
             .stdout(contains("fixtures/dump_inputs/markdown.md"))
-            .stdout(contains("fixtures/dump_inputs/subfolder/example.bin"))
             .stdout(contains("fixtures/dump_inputs/some_file.txt"));
 
         Ok(())
@@ -1774,23 +1900,119 @@ mod cli {
     #[test]
     fn test_dump_inputs_url() -> Result<()> {
         let mut cmd = main_command();
-        cmd.arg("--dump-inputs")
+        let result = cmd
+            .arg("--dump-inputs")
             .arg("https://example.com")
             .assert()
-            .success()
-            .stdout(contains("https://example.com"));
+            .success();
 
+        assert_lines_eq(result, vec!["https://example.com/"]);
         Ok(())
     }
 
     #[test]
     fn test_dump_inputs_path() -> Result<()> {
         let mut cmd = main_command();
-        cmd.arg("--dump-inputs")
-            .arg("fixtures")
+        let result = cmd
+            .arg("--dump-inputs")
+            .arg(fixtures_path().join("dump_inputs"))
+            .assert()
+            .success();
+
+        let base_path = fixtures_path().join("dump_inputs");
+        let expected_lines = [
+            "some_file.txt",
+            "subfolder/file2.md",
+            "subfolder/test.html",
+            "markdown.md",
+        ]
+        .iter()
+        .map(|p| path_str(&base_path, p))
+        .collect();
+
+        assert_lines_eq(result, expected_lines);
+        Ok(())
+    }
+
+    // Ensures that dumping stdin does not panic and results in an empty output
+    // as `stdin` is not a path
+    #[test]
+    fn test_dump_inputs_with_extensions() -> Result<()> {
+        let mut cmd = main_command();
+        let test_dir = fixtures_path().join("dump_inputs");
+
+        let output = cmd
+            .arg("--dump-inputs")
+            .arg("--extensions")
+            .arg("md,txt")
+            .arg(test_dir)
             .assert()
             .success()
-            .stdout(contains("fixtures"));
+            .get_output()
+            .stdout
+            .clone();
+
+        let mut actual_lines: Vec<String> = output
+            .lines()
+            .map(|line| line.unwrap().to_string())
+            .collect();
+        actual_lines.sort();
+
+        let base_path = fixtures_path().join("dump_inputs");
+        let mut expected_lines = vec![
+            path_str(&base_path, "some_file.txt"),
+            path_str(&base_path, "subfolder/file2.md"),
+            path_str(&base_path, "markdown.md"),
+        ];
+        expected_lines.sort();
+
+        assert_eq!(actual_lines, expected_lines);
+
+        // Verify example.bin is not included
+        for line in &actual_lines {
+            assert!(
+                !line.contains("example.bin"),
+                "Should not contain example.bin: {line}"
+            );
+        }
+
+        Ok(())
+    }
+
+    #[test]
+    fn test_dump_inputs_skip_hidden() -> Result<()> {
+        let test_dir = fixtures_path().join("hidden");
+
+        // Test default behavior (skip hidden)
+        main_command()
+            .arg("--dump-inputs")
+            .arg(&test_dir)
+            .assert()
+            .success()
+            .stdout(is_empty());
+
+        // Test with --hidden flag
+        main_command()
+            .arg("--dump-inputs")
+            .arg("--hidden")
+            .arg(test_dir)
+            .assert()
+            .success()
+            .stdout(contains(".hidden/file.md"));
+
+        Ok(())
+    }
+
+    #[test]
+    fn test_dump_inputs_individual_file() -> Result<()> {
+        let mut cmd = main_command();
+        let test_file = fixtures_path().join("TEST.md");
+
+        cmd.arg("--dump-inputs")
+            .arg(&test_file)
+            .assert()
+            .success()
+            .stdout(contains("fixtures/TEST.md"));
 
         Ok(())
     }
@@ -1798,49 +2020,140 @@ mod cli {
     #[test]
     fn test_dump_inputs_stdin() -> Result<()> {
         let mut cmd = main_command();
+
         cmd.arg("--dump-inputs")
             .arg("-")
             .assert()
             .success()
-            .stdout(contains("Stdin"));
+            .stdout(contains("<stdin>"));
 
         Ok(())
     }
 
+    #[test]
+    fn test_fragments_regression() {
+        let mut cmd = main_command();
+        let input = fixtures_path().join("FRAGMENT_REGRESSION.md");
+
+        cmd.arg("--include-fragments")
+            .arg("--verbose")
+            .arg(input)
+            .assert()
+            .failure();
+    }
+
     #[test]
     fn test_fragments() {
         let mut cmd = main_command();
         let input = fixtures_path().join("fragments");
 
+        let mut result = cmd
+            .arg("--include-fragments")
+            .arg("--verbose")
+            .arg(input)
+            .assert()
+            .failure();
+
+        let expected_successes = vec![
+            "fixtures/fragments/empty_dir",
+            "fixtures/fragments/empty_file#fragment", // XXX: is this a bug? a fragment in an empty file is being treated as valid
+            "fixtures/fragments/file1.md#code-heading",
+            "fixtures/fragments/file1.md#explicit-fragment",
+            "fixtures/fragments/file1.md#f%C3%BCnf-s%C3%9C%C3%9Fe-%C3%84pfel",
+            "fixtures/fragments/file1.md#f%C3%BCnf-s%C3%BC%C3%9Fe-%C3%A4pfel",
+            "fixtures/fragments/file1.md#fragment-1",
+            "fixtures/fragments/file1.md#fragment-2",
+            "fixtures/fragments/file1.md#IGNORE-CASING",
+            "fixtures/fragments/file1.md#kebab-case-fragment",
+            "fixtures/fragments/file1.md#kebab-case-fragment-1",
+            "fixtures/fragments/file1.md#lets-wear-a-hat-%C3%AAtre",
+            "fixtures/fragments/file2.md#",
+            "fixtures/fragments/file2.md#custom-id",
+            "fixtures/fragments/file2.md#fragment-1",
+            "fixtures/fragments/file2.md#top",
+            "fixtures/fragments/file.html#",
+            "fixtures/fragments/file.html#a-word",
+            "fixtures/fragments/file.html#in-the-beginning",
+            "fixtures/fragments/file.html#tangent%3A-kustomize",
+            "fixtures/fragments/file.html#top",
+            "fixtures/fragments/file.html#Upper-%C3%84%C3%96%C3%B6",
+            "fixtures/fragments/sub_dir",
+            "fixtures/fragments/zero.bin",
+            "fixtures/fragments/zero.bin#",
+            "fixtures/fragments/zero.bin#fragment",
+            "https://github.com/lycheeverse/lychee#table-of-contents",
+            "https://raw.githubusercontent.com/lycheeverse/lychee/master/fixtures/fragments/zero.bin",
+            "https://raw.githubusercontent.com/lycheeverse/lychee/master/fixtures/fragments/zero.bin#",
+            // zero.bin#fragment succeeds because fragment checking is skipped for this URL
+            "https://raw.githubusercontent.com/lycheeverse/lychee/master/fixtures/fragments/zero.bin#fragment",
+        ];
+
+        let expected_failures = vec![
+            "fixtures/fragments/sub_dir_non_existing_1",
+            "fixtures/fragments/sub_dir#non-existing-fragment-2",
+            "fixtures/fragments/sub_dir#a-link-inside-index-html-inside-sub-dir",
+            "fixtures/fragments/empty_dir#non-existing-fragment-3",
+            "fixtures/fragments/file2.md#missing-fragment",
+            "fixtures/fragments/sub_dir#non-existing-fragment-1",
+            "fixtures/fragments/sub_dir_non_existing_2",
+            "fixtures/fragments/file1.md#missing-fragment",
+            "fixtures/fragments/empty_dir#non-existing-fragment-4",
+            "fixtures/fragments/file.html#in-the-end",
+            "fixtures/fragments/file.html#in-THE-begiNNing",
+            "https://github.com/lycheeverse/lychee#non-existent-anchor",
+        ];
+
+        // the stdout/stderr format looks like this:
+        //
+        //     [ERROR] https://github.com/lycheeverse/lychee#non-existent-anchor | Cannot find fragment
+        //     [200] file:///home/rina/progs/lychee/fixtures/fragments/file.html#a-word
+        //
+        // errors are printed to both, but 200s are printed to stderr only.
+        // we take advantage of this to ensure that good URLs do not appear
+        // in stdout, and bad URLs do appear in stdout.
+        //
+        // also, a space or newline is appended to the URL to prevent
+        // incorrect matches where one URL is a prefix of another.
+        for good_url in &expected_successes {
+            // additionally checks that URL is within stderr to ensure that
+            // the URL is detected by lychee.
+            result = result
+                .stdout(contains(format!("{good_url} ")).not())
+                .stderr(contains(format!("{good_url}\n")));
+        }
+        for bad_url in &expected_failures {
+            result = result.stdout(contains(format!("{bad_url} ")));
+        }
+
+        let ok_num = expected_successes.len();
+        let err_num = expected_failures.len();
+        let total_num = ok_num + err_num;
+        result
+            .stdout(contains(format!("{ok_num} OK")))
+            // Failures because of missing fragments or failed binary body scan
+            .stdout(contains(format!("{err_num} Errors")))
+            .stdout(contains(format!("{total_num} Total")));
+    }
+
+    #[test]
+    fn test_fragments_when_accept_error_status_codes() {
+        let mut cmd = main_command();
+        let input = fixtures_path().join("TEST_FRAGMENT_ERR_CODE.md");
+
+        // it's common for user to accept 429, but let's test with 404 since
+        // triggering 429 may annoy the server
         cmd.arg("--verbose")
+            .arg("--accept=200,404")
             .arg("--include-fragments")
             .arg(input)
             .assert()
-            .failure()
-            .stderr(contains("fixtures/fragments/file1.md#fragment-1"))
-            .stderr(contains("fixtures/fragments/file1.md#fragment-2"))
-            .stderr(contains("fixtures/fragments/file1.md#code-heading"))
-            .stderr(contains("fixtures/fragments/file2.md#custom-id"))
-            .stderr(contains("fixtures/fragments/file1.md#missing-fragment"))
-            .stderr(contains("fixtures/fragments/file2.md#fragment-1"))
-            .stderr(contains("fixtures/fragments/file1.md#kebab-case-fragment"))
+            .success()
             .stderr(contains(
-                "fixtures/fragments/file1.md#lets-wear-a-hat-%C3%AAtre",
+                "https://en.wikipedia.org/wiki/Should404#ignore-fragment",
             ))
-            .stderr(contains("fixtures/fragments/file2.md#missing-fragment"))
-            .stderr(contains("fixtures/fragments/empty_file#fragment"))
-            .stderr(contains("fixtures/fragments/file.html#a-word"))
-            .stderr(contains("fixtures/fragments/file.html#in-the-beginning"))
-            .stderr(contains("fixtures/fragments/file.html#in-the-end"))
-            .stderr(contains(
-                "fixtures/fragments/file1.md#kebab-case-fragment-1",
-            ))
-            .stderr(contains("fixtures/fragments/file.html#top"))
-            .stderr(contains("fixtures/fragments/file2.md#top"))
-            .stdout(contains("25 Total"))
-            .stdout(contains("21 OK"))
-            // 4 failures because of missing fragments
-            .stdout(contains("4 Errors"));
+            .stdout(contains("0 Errors"))
+            .stdout(contains("1 OK"))
+            .stdout(contains("1 Total"));
     }
 
     #[test]
@@ -1856,6 +2169,24 @@ mod cli {
             .stdout(contains("0 Errors"));
     }
 
+    #[test]
+    fn test_fragments_fallback_extensions() {
+        let mut cmd = main_command();
+        let input = fixtures_path().join("fragments-fallback-extensions");
+
+        cmd.arg("--include-fragments")
+            .arg("--fallback-extensions=html")
+            .arg("--no-progress")
+            .arg("--offline")
+            .arg("-v")
+            .arg(input)
+            .assert()
+            .failure()
+            .stdout(contains("3 Total"))
+            .stdout(contains("1 OK"))
+            .stdout(contains("2 Errors"));
+    }
+
     /// Test relative paths
     ///
     /// Imagine a web server hosting a site with the following structure:
@@ -1943,4 +2274,352 @@ mod cli {
 
         Ok(())
     }
+
+    #[tokio::test]
+    async fn test_no_header_set_on_input() -> Result<()> {
+        let mut cmd = main_command();
+        let server = wiremock::MockServer::start().await;
+        server
+            .register(
+                wiremock::Mock::given(wiremock::matchers::method("GET"))
+                    .respond_with(wiremock::ResponseTemplate::new(200))
+                    .expect(1),
+            )
+            .await;
+
+        cmd.arg("--verbose").arg(server.uri()).assert().success();
+
+        let received_requests = server.received_requests().await.unwrap();
+        assert_eq!(received_requests.len(), 1);
+
+        let received_request = &received_requests[0];
+        assert_eq!(received_request.method, Method::GET);
+        assert_eq!(received_request.url.path(), "/");
+
+        // Make sure the request does not contain the custom header
+        assert!(!received_request.headers.contains_key("X-Foo"));
+        Ok(())
+    }
+
+    #[tokio::test]
+    async fn test_header_set_on_input() -> Result<()> {
+        let mut cmd = main_command();
+        let server = wiremock::MockServer::start().await;
+        server
+            .register(
+                wiremock::Mock::given(wiremock::matchers::method("GET"))
+                    .and(wiremock::matchers::header("X-Foo", "Bar"))
+                    .respond_with(wiremock::ResponseTemplate::new(200))
+                    // We expect the mock to be called exactly least once.
+                    .expect(1)
+                    .named("GET expecting custom header"),
+            )
+            .await;
+
+        cmd.arg("--verbose")
+            .arg("--header")
+            .arg("X-Foo: Bar")
+            .arg(server.uri())
+            .assert()
+            .success();
+
+        // Check that the server received the request with the header
+        server.verify().await;
+        Ok(())
+    }
+
+    #[tokio::test]
+    async fn test_multi_header_set_on_input() -> Result<()> {
+        let mut cmd = main_command();
+        let server = wiremock::MockServer::start().await;
+        server
+            .register(
+                wiremock::Mock::given(wiremock::matchers::method("GET"))
+                    .and(wiremock::matchers::header("X-Foo", "Bar"))
+                    .and(wiremock::matchers::header("X-Bar", "Baz"))
+                    .respond_with(wiremock::ResponseTemplate::new(200))
+                    // We expect the mock to be called exactly least once.
+                    .expect(1)
+                    .named("GET expecting custom header"),
+            )
+            .await;
+
+        cmd.arg("--verbose")
+            .arg("--header")
+            .arg("X-Foo: Bar")
+            .arg("--header")
+            .arg("X-Bar: Baz")
+            .arg(server.uri())
+            .assert()
+            .success();
+
+        // Check that the server received the request with the header
+        server.verify().await;
+        Ok(())
+    }
+
+    #[tokio::test]
+    async fn test_header_set_in_config() -> Result<()> {
+        let mut cmd = main_command();
+        let server = wiremock::MockServer::start().await;
+        server
+            .register(
+                wiremock::Mock::given(wiremock::matchers::method("GET"))
+                    .and(wiremock::matchers::header("X-Foo", "Bar"))
+                    .and(wiremock::matchers::header("X-Bar", "Baz"))
+                    .respond_with(wiremock::ResponseTemplate::new(200))
+                    // We expect the mock to be called exactly least once.
+                    .expect(1)
+                    .named("GET expecting custom header"),
+            )
+            .await;
+
+        let config = fixtures_path().join("configs").join("headers.toml");
+        cmd.arg("--verbose")
+            .arg("--config")
+            .arg(config)
+            .arg(server.uri())
+            .assert()
+            .success();
+
+        // Check that the server received the request with the header
+        server.verify().await;
+        Ok(())
+    }
+
+    #[test]
+    fn test_sorted_error_output() -> Result<()> {
+        let test_files = ["TEST_GITHUB_404.md", "TEST_INVALID_URLS.html"];
+
+        let test_urls = [
+            "https://httpbin.org/status/404",
+            "https://httpbin.org/status/500",
+            "https://httpbin.org/status/502",
+        ];
+
+        let cmd = &mut main_command()
+            .arg("--format")
+            .arg("compact")
+            .arg(fixtures_path().join(test_files[1]))
+            .arg(fixtures_path().join(test_files[0]))
+            .assert()
+            .failure()
+            .code(2);
+
+        let output = String::from_utf8_lossy(&cmd.get_output().stdout);
+        let mut position: usize = 0;
+
+        // Check that the input sources are sorted
+        for file in test_files {
+            assert!(output.contains(file));
+
+            let next_position = output.find(file).unwrap();
+
+            assert!(next_position > position);
+            position = next_position;
+        }
+
+        position = 0;
+
+        // Check that the responses are sorted
+        for url in test_urls {
+            assert!(output.contains(url));
+
+            let next_position = output.find(url).unwrap();
+
+            assert!(next_position > position);
+            position = next_position;
+        }
+
+        Ok(())
+    }
+
+    #[test]
+    fn test_extract_url_ending_with_period_file() {
+        let test_path = fixtures_path().join("LINK_PERIOD.html");
+
+        let mut cmd = main_command();
+        cmd.arg("--dump")
+            .arg(test_path)
+            .assert()
+            .success()
+            .stdout(contains("https://www.example.com/smth."));
+    }
+
+    #[tokio::test]
+    async fn test_extract_url_ending_with_period_webserver() {
+        let mut cmd = main_command();
+        let body = r#"<a href="https://www.example.com/smth.">link</a>"#;
+        let mock_server = mock_response!(body);
+
+        cmd.arg("--dump")
+            .arg(mock_server.uri())
+            .assert()
+            .success()
+            .stdout(contains("https://www.example.com/smth."));
+    }
+
+    #[test]
+    fn test_wikilink_extract_when_specified() {
+        let test_path = fixtures_path().join("TEST_WIKI.md");
+
+        let mut cmd = main_command();
+        cmd.arg("--dump")
+            .arg("--include-wikilinks")
+            .arg(test_path)
+            .assert()
+            .success()
+            .stdout(contains("LycheeWikilink"));
+    }
+
+    #[test]
+    fn test_wikilink_dont_extract_when_not_specified() {
+        let test_path = fixtures_path().join("TEST_WIKI.md");
+
+        let mut cmd = main_command();
+        cmd.arg("--dump")
+            .arg(test_path)
+            .assert()
+            .success()
+            .stdout(is_empty());
+    }
+
+    #[test]
+    fn test_index_files_default() {
+        let input = fixtures_path().join("filechecker/dir_links.md");
+
+        // the dir links in this file all exist.
+        main_command()
+            .arg(&input)
+            .arg("--verbose")
+            .assert()
+            .success();
+
+        // ... but checking fragments will find none, because dirs
+        // have no fragments and no index file given.
+        let dir_links_with_fragment = 2;
+        main_command()
+            .arg(&input)
+            .arg("--include-fragments")
+            .assert()
+            .failure()
+            .stdout(contains("Cannot find fragment").count(dir_links_with_fragment))
+            .stdout(contains("#").count(dir_links_with_fragment));
+    }
+
+    #[test]
+    fn test_index_files_specified() {
+        let input = fixtures_path().join("filechecker/dir_links.md");
+
+        // passing `--index-files index.html` should reject all links
+        // to /empty_dir because it doesn't have the index file
+        let result = main_command()
+            .arg(input)
+            .arg("--index-files")
+            .arg("index.html")
+            .arg("--verbose")
+            .assert()
+            .failure();
+
+        let empty_dir_links = 2;
+        let index_dir_links = 2;
+        result
+            .stdout(contains("Cannot find index file").count(empty_dir_links))
+            .stdout(contains("/empty_dir").count(empty_dir_links))
+            .stdout(contains(format!("{index_dir_links} OK")));
+    }
+
+    #[test]
+    fn test_index_files_dot_in_list() {
+        let input = fixtures_path().join("filechecker/dir_links.md");
+
+        // passing `.` in the index files list should accept a directory
+        // even if no other index file is found.
+        main_command()
+            .arg(&input)
+            .arg("--index-files")
+            .arg("index.html,.")
+            .assert()
+            .success()
+            .stdout(contains("4 OK"));
+
+        // checking fragments will accept the index_dir#fragment link,
+        // but reject empty_dir#fragment because empty_dir doesn’t have
+        // index.html.
+        main_command()
+            .arg(&input)
+            .arg("--index-files")
+            .arg("index.html,.")
+            .arg("--include-fragments")
+            .assert()
+            .failure()
+            .stdout(contains("Cannot find fragment").count(1))
+            .stdout(contains("empty_dir#fragment").count(1))
+            .stdout(contains("index_dir#fragment").count(0))
+            .stdout(contains("3 OK"));
+    }
+
+    #[test]
+    fn test_index_files_empty_list() {
+        let input = fixtures_path().join("filechecker/dir_links.md");
+
+        // passing an empty list to --index-files should reject /all/
+        // directory links.
+        let result = main_command()
+            .arg(input)
+            .arg("--index-files")
+            .arg("")
+            .assert()
+            .failure();
+
+        let num_dir_links = 4;
+        result
+            .stdout(contains("Cannot find index file").count(num_dir_links))
+            .stdout(contains("0 OK"));
+    }
+
+    #[test]
+    fn test_skip_binary_input() {
+        // A path containing a binary file
+        let inputs = fixtures_path().join("invalid_utf8");
+
+        // Run the command with the binary input
+        let mut cmd = main_command();
+        let result = cmd
+            .arg("--verbose")
+            .arg(&inputs)
+            .assert()
+            .success()
+            .stdout(contains("1 Total"))
+            .stdout(contains("1 OK"))
+            .stdout(contains("0 Errors"));
+
+        result
+            .stderr(contains(format!(
+                "Skipping file with invalid UTF-8 content: {}",
+                inputs.join("invalid_utf8.txt").display()
+            )))
+            .stderr(contains("https://example.com/"));
+    }
+
+    /// Checks that the `--dump-inputs` command does not panic
+    /// when given a path that contains invalid UTF-8 characters.
+    ///
+    /// The command should still succeed and output the paths of the files it
+    /// found, including those with invalid UTF-8, which will be skipped during
+    /// processing.
+    #[test]
+    fn test_dump_invalid_utf8_inputs() {
+        // A path containing a binary file
+        let inputs = fixtures_path().join("invalid_utf8");
+
+        // Run the command with the binary input
+        let mut cmd = main_command();
+        cmd.arg("--dump-inputs")
+            .arg(inputs)
+            .assert()
+            .success()
+            .stdout(contains("fixtures/invalid_utf8/index.html"))
+            .stdout(contains("fixtures/invalid_utf8/invalid_utf8.txt"));
+    }
 }
