@@ -1,12 +1,19 @@
-mod excludes;
-mod includes;
+mod regex_filter;
 
-use once_cell::sync::Lazy;
 use regex::RegexSet;
 use std::collections::HashSet;
+use std::sync::LazyLock;
 
-pub use excludes::Excludes;
-pub use includes::Includes;
+/// Include configuration for the link checker.
+/// You can include links based on regex patterns.
+pub type Includes = regex_filter::RegexFilter;
+
+/// Exclude configuration for the link checker.
+/// You can ignore links based on regex patterns.
+pub type Excludes = regex_filter::RegexFilter;
+
+/// You can exclude paths and files based on regex patterns.
+pub type PathExcludes = regex_filter::RegexFilter;
 
 use crate::Uri;
 
@@ -14,23 +21,24 @@ use crate::Uri;
 /// These domains are explicitly defined by RFC 2606, section 3 Reserved Example
 /// Second Level Domain Names for describing example cases and should not be
 /// dereferenced as they should not have content.
-static EXAMPLE_DOMAINS: Lazy<HashSet<&'static str>> =
-    Lazy::new(|| HashSet::from_iter(["example.com", "example.org", "example.net", "example.edu"]));
+static EXAMPLE_DOMAINS: LazyLock<HashSet<&'static str>> = LazyLock::new(|| {
+    HashSet::from_iter(["example.com", "example.org", "example.net", "example.edu"])
+});
 
 #[cfg(all(not(test), not(feature = "check_example_domains")))]
 /// We also exclude the example TLDs in section 2 of the same RFC.
 /// This exclusion gets subsumed by the `check_example_domains` feature.
-static EXAMPLE_TLDS: Lazy<HashSet<&'static str>> =
-    Lazy::new(|| HashSet::from_iter([".test", ".example", ".invalid", ".localhost"]));
+static EXAMPLE_TLDS: LazyLock<HashSet<&'static str>> =
+    LazyLock::new(|| HashSet::from_iter([".test", ".example", ".invalid", ".localhost"]));
 
 // Allow usage of example domains in tests
 #[cfg(any(test, feature = "check_example_domains"))]
-static EXAMPLE_DOMAINS: Lazy<HashSet<&'static str>> = Lazy::new(HashSet::new);
+static EXAMPLE_DOMAINS: LazyLock<HashSet<&'static str>> = LazyLock::new(HashSet::new);
 
 #[cfg(any(test, feature = "check_example_domains"))]
-static EXAMPLE_TLDS: Lazy<HashSet<&'static str>> = Lazy::new(HashSet::new);
+static EXAMPLE_TLDS: LazyLock<HashSet<&'static str>> = LazyLock::new(HashSet::new);
 
-static UNSUPPORTED_DOMAINS: Lazy<HashSet<&'static str>> = Lazy::new(|| {
+static UNSUPPORTED_DOMAINS: LazyLock<HashSet<&'static str>> = LazyLock::new(|| {
     HashSet::from_iter([
         // Twitter requires an account to view tweets
         // https://news.ycombinator.com/item?id=36540957
@@ -40,18 +48,19 @@ static UNSUPPORTED_DOMAINS: Lazy<HashSet<&'static str>> = Lazy::new(|| {
 
 /// Pre-defined exclusions for known false-positives
 const FALSE_POSITIVE_PAT: &[&str] = &[
-    r"^https?://schemas.openxmlformats.org",
-    r"^https?://schemas.zune.net",
-    r"^https?://www.w3.org/1999/xhtml",
-    r"^https?://www.w3.org/1999/xlink",
-    r"^https?://www.w3.org/2000/svg",
-    r"^https?://ogp.me/ns#",
-    r"^https?://schemas.microsoft.com",
-    r"^https?://(.*)/xmlrpc.php$",
+    r"^https?://schemas\.openxmlformats\.org",
+    r"^https?://schemas\.microsoft\.com",
+    r"^https?://schemas\.zune\.net",
+    r"^https?://www\.w3\.org/1999/xhtml",
+    r"^https?://www\.w3\.org/1999/xlink",
+    r"^https?://www\.w3\.org/2000/svg",
+    r"^https?://www\.w3\.org/2001/XMLSchema-instance",
+    r"^https?://ogp\.me/ns#",
+    r"^https?://(.*)/xmlrpc\.php$",
 ];
 
-static FALSE_POSITIVE_SET: Lazy<RegexSet> =
-    Lazy::new(|| regex::RegexSet::new(FALSE_POSITIVE_PAT).expect("Failed to create RegexSet"));
+static FALSE_POSITIVE_SET: LazyLock<RegexSet> =
+    LazyLock::new(|| regex::RegexSet::new(FALSE_POSITIVE_PAT).expect("Failed to create RegexSet"));
 
 #[inline]
 #[must_use]
@@ -252,14 +261,13 @@ impl Filter {
 
 #[cfg(test)]
 mod tests {
-    use regex::RegexSet;
     use reqwest::Url;
     use url::Host;
 
     use super::{Excludes, Filter, Includes};
     use crate::{
-        test_utils::{mail, website},
         Uri,
+        test_utils::{mail, website},
     };
 
     // Note: the standard library, as of Rust stable 1.47.0, does not expose
@@ -351,9 +359,7 @@ mod tests {
 
     #[test]
     fn test_overwrite_false_positives() {
-        let includes = Includes {
-            regex: RegexSet::new([r"http://www.w3.org/1999/xhtml"]).unwrap(),
-        };
+        let includes = Includes::new([r"http://www.w3.org/1999/xhtml"]).unwrap();
         let filter = Filter {
             includes: Some(includes),
             ..Filter::default()
@@ -363,9 +369,7 @@ mod tests {
 
     #[test]
     fn test_include_regex() {
-        let includes = Includes {
-            regex: RegexSet::new([r"foo.example.com"]).unwrap(),
-        };
+        let includes = Includes::new([r"foo.example.com"]).unwrap();
         let filter = Filter {
             includes: Some(includes),
             ..Filter::default()
@@ -402,9 +406,8 @@ mod tests {
 
     #[test]
     fn test_exclude_regex() {
-        let excludes = Excludes {
-            regex: RegexSet::new([r"github.com", r"[a-z]+\.(org|net)", r"@example.com"]).unwrap(),
-        };
+        let excludes =
+            Excludes::new([r"github.com", r"[a-z]+\.(org|net)", r"@example.com"]).unwrap();
         let filter = Filter {
             excludes: Some(excludes),
             ..Filter::default()
@@ -419,12 +422,8 @@ mod tests {
     }
     #[test]
     fn test_exclude_include_regex() {
-        let includes = Includes {
-            regex: RegexSet::new([r"foo.example.com"]).unwrap(),
-        };
-        let excludes = Excludes {
-            regex: RegexSet::new([r"example.com"]).unwrap(),
-        };
+        let includes = Includes::new([r"foo.example.com"]).unwrap();
+        let excludes = Excludes::new([r"example.com"]).unwrap();
         let filter = Filter {
             includes: Some(includes),
             excludes: Some(excludes),
