@@ -1,6 +1,10 @@
+use super::file::FileExtensions;
+use super::resolver::UrlContentResolver;
+use crate::filter::PathExcludes;
 use crate::types::FileType;
-use crate::{utils, ErrorKind, Result};
+use crate::{ErrorKind, Result};
 use async_stream::try_stream;
+use futures::StreamExt;
 use futures::stream::Stream;
 use glob::glob_with;
 use ignore::WalkBuilder;
@@ -10,16 +14,10 @@ use shellexpand::tilde;
 use std::fmt::Display;
 use std::fs;
 use std::path::{Path, PathBuf};
-use tokio::io::{stdin, AsyncReadExt};
+use tokio::io::{AsyncReadExt, stdin};
 
 const STDIN: &str = "-";
 
-// Check the extension of the given path against the list of known/accepted
-// file extensions
-fn valid_extension(p: &Path) -> bool {
-    matches!(FileType::from(p), FileType::Markdown | FileType::Html)
-}
-
 #[derive(Debug)]
 /// Encapsulates the content for a given input
 pub struct InputContent {
@@ -48,8 +46,17 @@ impl TryFrom<&PathBuf> for InputContent {
     type Error = crate::ErrorKind;
 
     fn try_from(path: &PathBuf) -> std::result::Result<Self, Self::Error> {
-        let input =
-            fs::read_to_string(path).map_err(|e| ErrorKind::ReadFileInput(e, path.clone()))?;
+        let input = match fs::read_to_string(path) {
+            Ok(content) => content,
+            Err(e) if e.kind() == std::io::ErrorKind::InvalidData => {
+                log::warn!(
+                    "Skipping file with invalid UTF-8 content: {}",
+                    path.display()
+                );
+                return Err(ErrorKind::ReadFileInput(e, path.clone()));
+            }
+            Err(e) => return Err(ErrorKind::ReadFileInput(e, path.clone())),
+        };
 
         Ok(Self {
             source: InputSource::String(input.clone()),
@@ -111,8 +118,6 @@ pub struct Input {
     pub source: InputSource,
     /// Hint to indicate which extractor to use
     pub file_type_hint: Option<FileType>,
-    /// Excluded paths that will be skipped when reading content
-    pub excluded_paths: Option<Vec<PathBuf>>,
 }
 
 impl Input {
@@ -125,36 +130,35 @@ impl Input {
     /// Returns an error if the input does not exist (i.e. invalid path)
     /// and the input cannot be parsed as a URL.
     pub fn new(
-        value: &str,
+        input: &str,
         file_type_hint: Option<FileType>,
         glob_ignore_case: bool,
-        excluded_paths: Option<Vec<PathBuf>>,
     ) -> Result<Self> {
-        let source = if value == STDIN {
+        let source = if input == STDIN {
             InputSource::Stdin
         } else {
             // We use [`reqwest::Url::parse`] because it catches some other edge cases that [`http::Request:builder`] does not
             // This could be improved with further refinement.
-            match Url::parse(value) {
+            match Url::parse(input) {
                 // Weed out non-http schemes, including Windows drive specifiers, which will be successfully parsed by the Url crate
                 Ok(url) if url.scheme() == "http" || url.scheme() == "https" => {
                     InputSource::RemoteUrl(Box::new(url))
                 }
                 Ok(_) => {
                     // URL parsed successfully, but it's not http or https
-                    return Err(ErrorKind::InvalidFile(PathBuf::from(value)));
+                    return Err(ErrorKind::InvalidFile(PathBuf::from(input)));
                 }
                 _ => {
                     // this seems to be the only way to determine if this is a glob pattern
-                    let is_glob = glob::Pattern::escape(value) != value;
+                    let is_glob = glob::Pattern::escape(input) != input;
 
                     if is_glob {
                         InputSource::FsGlob {
-                            pattern: value.to_owned(),
+                            pattern: input.to_owned(),
                             ignore_case: glob_ignore_case,
                         }
                     } else {
-                        let path = PathBuf::from(value);
+                        let path = PathBuf::from(input);
 
                         // On Windows, a filepath can never be mistaken for a url because Windows filepaths use \ and urls use /
                         #[cfg(windows)]
@@ -169,7 +173,7 @@ impl Input {
                         #[cfg(unix)]
                         if path.exists() {
                             InputSource::FsPath(path)
-                        } else if value.starts_with('~') || value.starts_with('.') {
+                        } else if input.starts_with('~') || input.starts_with('.') {
                             // The path is not valid, but it might be a valid URL
                             // Check if the path starts with a tilde or a dot
                             // and exit early if it does
@@ -181,7 +185,7 @@ impl Input {
                             // by prefixing it with a `http://` scheme.
                             // Curl also uses http (i.e. not https), see
                             // https://github.com/curl/curl/blob/70ac27604a2abfa809a7b2736506af0da8c3c8a9/lib/urlapi.c#L1104-L1124
-                            let url = Url::parse(&format!("http://{value}")).map_err(|e| {
+                            let url = Url::parse(&format!("http://{input}")).map_err(|e| {
                                 ErrorKind::ParseUrl(e, "Input is not a valid URL".to_string())
                             })?;
                             InputSource::RemoteUrl(Box::new(url))
@@ -193,12 +197,133 @@ impl Input {
         Ok(Self {
             source,
             file_type_hint,
-            excluded_paths,
         })
     }
 
+    /// Convenience constructor with sane defaults
+    ///
+    /// # Errors
+    ///
+    /// Returns an error if the input does not exist (i.e. invalid path)
+    /// and the input cannot be parsed as a URL.
+    pub fn from_value(value: &str) -> Result<Self> {
+        Self::new(value, None, false)
+    }
+
+    /// Convenience constructor
+    #[must_use]
+    pub const fn from_input_source(source: InputSource) -> Self {
+        Self {
+            source,
+            file_type_hint: None,
+        }
+    }
+
+    /// Get all input sources for content processing.
+    ///
+    /// This method returns a stream of input sources for the given input, taking into
+    /// account the matching file extensions and respecting exclusions.
+    ///
+    /// This can be used for retrieving all inputs which lychee would check for
+    /// links.
+    ///
+    /// # Returns
+    ///
+    /// Returns a stream of `Result<InputSource>` for all matching input sources.
+    ///
+    /// # Errors
+    ///
+    /// Will return errors for file system operations or glob pattern issues
+    pub fn get_input_sources<'a>(
+        &'a self,
+        file_extensions: FileExtensions,
+        skip_hidden: bool,
+        skip_gitignored: bool,
+        excluded_paths: &'a PathExcludes,
+    ) -> impl Stream<Item = Result<InputSource>> + 'a {
+        try_stream! {
+            match &self.source {
+                InputSource::RemoteUrl(url) => {
+                    // Yield the remote URL as an input source
+                    yield InputSource::RemoteUrl(url.clone());
+                },
+                InputSource::FsGlob { pattern, ignore_case } => {
+                    // For glob patterns, we expand the pattern and yield matching paths
+                    let glob_expanded = tilde(pattern).to_string();
+                    let mut match_opts = glob::MatchOptions::new();
+                    match_opts.case_sensitive = !ignore_case;
+
+                    for entry in glob_with(&glob_expanded, match_opts)? {
+                        match entry {
+                            Ok(path) => {
+                                // Skip directories or files that don't match extensions
+                                if path.is_dir() {
+                                    continue;
+                                }
+                                if Self::is_excluded_path(&path, excluded_paths) {
+                                    continue;
+                                }
+
+                                // Check if it matches one of our file extensions
+                                if file_extensions_match(&path, &file_extensions) {
+                                    yield InputSource::FsPath(path);
+                                }
+                            }
+                            Err(e) => {
+                                eprintln!("Error in glob pattern: {e:?}");
+                            }
+                        }
+                    }
+                },
+                InputSource::FsPath(path) => {
+                    if path.is_dir() {
+
+                        for entry in Input::walk_entries(
+                            path,
+                            file_extensions,
+                            skip_hidden,
+                            skip_gitignored,
+                        )?
+                        {
+                            let entry = entry?;
+                            if Self::is_excluded_path(entry.path(), excluded_paths) {
+                                continue;
+                            }
+                            match entry.file_type() {
+                                None => continue,
+                                Some(file_type) => {
+                                    if !file_type.is_file() {
+                                        continue;
+                                    }
+                                }
+                            }
+
+                            yield InputSource::FsPath(entry.path().to_path_buf());
+                        }
+                    } else {
+                        // For individual files, yield if not excluded and matches extensions
+                        if !Self::is_excluded_path(path, excluded_paths) && file_extensions_match(path, &file_extensions) {
+                            yield InputSource::FsPath(path.clone());
+                        }
+                    }
+                },
+                InputSource::Stdin => {
+                    // Yield stdin as an input source
+                    yield InputSource::Stdin;
+                },
+                InputSource::String(_) => {
+                    // Yield the string source
+                    yield self.source.clone();
+                }
+            }
+        }
+    }
+
     /// Retrieve the contents from the input
     ///
+    /// If the input is a path, only search through files that match the given
+    /// file extensions.
+    ///
     /// # Errors
     ///
     /// Returns an error if the contents can not be retrieved
@@ -209,71 +334,104 @@ impl Input {
         skip_missing: bool,
         skip_hidden: bool,
         skip_gitignored: bool,
+        // If `Input` is a file path, try the given file extensions in order.
+        // Stop on the first match.
+        file_extensions: FileExtensions,
+        resolver: UrlContentResolver,
+        excluded_paths: PathExcludes,
     ) -> impl Stream<Item = Result<InputContent>> {
         try_stream! {
+            // Handle special cases first
             match self.source {
-                InputSource::RemoteUrl(ref url) => {
-                    let content = Self::url_contents(url).await;
+                InputSource::RemoteUrl(url) => {
+                    let content = resolver.url_contents(*url).await;
                     match content {
                         Err(_) if skip_missing => (),
                         Err(e) => Err(e)?,
                         Ok(content) => yield content,
                     }
-                },
-                InputSource::FsGlob {
-                    ref pattern,
-                    ignore_case,
-                } => {
-                    for await content in self.glob_contents(pattern, ignore_case) {
-                        let content = content?;
-                        yield content;
-                    }
-                }
-                InputSource::FsPath(ref path) => {
-                    if path.is_dir() {
-                        for entry in WalkBuilder::new(path).standard_filters(skip_gitignored).hidden(skip_hidden).build() {
-                            let entry = entry?;
-
-                            if self.is_excluded_path(&entry.path().to_path_buf()) {
-                                continue;
-                            }
-
-                            match entry.file_type() {
-                                None => continue,
-                                Some(file_type) => {
-                                    if !file_type.is_file() || !valid_extension(entry.path()) {
-                                        continue;
-                                    }
-                                }
-                            };
-
-                            let content = Self::path_content(entry.path()).await?;
-                            yield content
-                        }
-                    } else {
-                        if self.is_excluded_path(path) {
-                            return;
-                        }
-                        let content = Self::path_content(path).await;
-                        match content {
-                            Err(_) if skip_missing => (),
-                            Err(e) => Err(e)?,
-                            Ok(content) => yield content,
-                        };
-                    }
+                    return;
                 },
                 InputSource::Stdin => {
                     let content = Self::stdin_content(self.file_type_hint).await?;
                     yield content;
+                    return;
                 },
                 InputSource::String(ref s) => {
                     let content = Self::string_content(s, self.file_type_hint);
                     yield content;
+                    return;
                 },
+                _ => {}
+            }
+
+            // Handle FsPath and FsGlob sources
+            // We can use `get_input_sources` to get the input sources, which will handle
+            // filtering by file extensions and exclusions
+            let mut sources_stream = Box::pin(self.get_input_sources(file_extensions, skip_hidden, skip_gitignored, &excluded_paths));
+
+            while let Some(source_result) = sources_stream.next().await {
+                match source_result {
+                    Ok(source) => {
+                        match source {
+                            InputSource::FsPath(path) => {
+                                // Process the actual file path
+                                let content = Self::path_content(&path).await;
+                                match content {
+                                    Err(_) if skip_missing => (),
+                                    Err(e) if matches!(&e, ErrorKind::ReadFileInput(io_err, _) if io_err.kind() == std::io::ErrorKind::InvalidData) => {
+                                        // If the file contains invalid UTF-8 (e.g. binary), we skip it
+                                        log::warn!("Skipping file with invalid UTF-8 content: {}", path.display());
+                                    },
+                                    Err(e) => Err(e)?,
+                                    Ok(content) => yield content,
+                                }
+                            },
+                            InputSource::RemoteUrl(url) => {
+                                let content = resolver.url_contents(*url).await;
+                                match content {
+                                    Err(_) if skip_missing => (),
+                                    Err(e) => Err(e)?,
+                                    Ok(content) => yield content,
+                                }
+                            },
+                            InputSource::Stdin => {
+                                let content = Self::stdin_content(self.file_type_hint).await?;
+                                yield content;
+                            },
+                            InputSource::String(s) => {
+                                let content = Self::string_content(&s, self.file_type_hint);
+                                yield content;
+                            },
+                            InputSource::FsGlob { .. } => {
+                                unreachable!("This shouldn't happen as `get_input_sources` expands the glob patterns");
+                            }
+                        }
+                    },
+                    Err(e) => Err(e)?,
+                }
             }
         }
     }
 
+    /// Create a `WalkBuilder` for directory traversal with consistent settings
+    fn walk_entries(
+        path: &Path,
+        file_extensions: FileExtensions,
+        skip_hidden: bool,
+        skip_gitignored: bool,
+    ) -> Result<ignore::Walk> {
+        Ok(WalkBuilder::new(path)
+            // Enable standard filters if `skip_gitignored `is true.
+            // This will skip files ignored by `.gitignore` and other VCS ignore files.
+            .standard_filters(skip_gitignored)
+            // Override hidden file behavior to be controlled by the separate skip_hidden parameter
+            .hidden(skip_hidden)
+            // Configure the file types filter to only include files with matching extensions
+            .types(file_extensions.try_into()?)
+            .build())
+    }
+
     /// Retrieve all sources from this input. The output depends on the type of
     /// input:
     ///
@@ -285,90 +443,63 @@ impl Input {
     /// # Errors
     ///
     /// Returns an error if the globbing fails with the expanded pattern.
-    pub fn get_sources(self) -> impl Stream<Item = Result<String>> {
+    pub fn get_sources(
+        self,
+        file_extensions: FileExtensions,
+        skip_hidden: bool,
+        skip_gitignored: bool,
+        excluded_paths: &PathExcludes,
+    ) -> impl Stream<Item = Result<String>> {
         try_stream! {
             match self.source {
                 InputSource::RemoteUrl(url) => yield url.to_string(),
-                InputSource::FsGlob { pattern, ignore_case } => {
+                InputSource::FsGlob {
+                    ref pattern,
+                    ignore_case,
+                } => {
                     let glob_expanded = tilde(&pattern).to_string();
                     let mut match_opts = glob::MatchOptions::new();
-
                     match_opts.case_sensitive = !ignore_case;
-
                     for entry in glob_with(&glob_expanded, match_opts)? {
                         match entry {
-                            Ok(path) => yield path.to_string_lossy().to_string(),
-                            Err(e) => eprintln!("{e:?}")
+                            Ok(path) => {
+                                if !Self::is_excluded_path(&path, excluded_paths) {
+                                    yield path.to_string_lossy().to_string();
+                                }
+                            },
+                            Err(e) => eprintln!("{e:?}"),
                         }
                     }
-                },
-                InputSource::FsPath(path) => yield path.to_string_lossy().to_string(),
-                InputSource::Stdin => yield "Stdin".into(),
-                InputSource::String(_) => yield "Raw String".into(),
-            }
-        }
-    }
-
-    async fn url_contents(url: &Url) -> Result<InputContent> {
-        // Assume HTML for default paths
-        let file_type = if url.path().is_empty() || url.path() == "/" {
-            FileType::Html
-        } else {
-            FileType::from(url.as_str())
-        };
-
-        let res = reqwest::get(url.clone())
-            .await
-            .map_err(ErrorKind::NetworkRequest)?;
-        let input_content = InputContent {
-            source: InputSource::RemoteUrl(Box::new(url.clone())),
-            file_type,
-            content: res.text().await.map_err(ErrorKind::ReadResponseBody)?,
-        };
-
-        Ok(input_content)
-    }
-
-    fn glob_contents(
-        &self,
-        pattern: &str,
-        ignore_case: bool,
-    ) -> impl Stream<Item = Result<InputContent>> + '_ {
-        let glob_expanded = tilde(&pattern).to_string();
-        let mut match_opts = glob::MatchOptions::new();
-
-        match_opts.case_sensitive = !ignore_case;
-
-        try_stream! {
-            for entry in glob_with(&glob_expanded, match_opts)? {
-                match entry {
-                    Ok(path) => {
-                        // Directories can have a suffix which looks like
-                        // a file extension (like `foo.html`). This can lead to
-                        // unexpected behavior with glob patterns like
-                        // `**/*.html`. Therefore filter these out.
-                        // See <https://github.com/lycheeverse/lychee/pull/262#issuecomment-913226819>
-                        if path.is_dir() {
-                            continue;
-                        }
-                        if self.is_excluded_path(&path) {
-                            continue;
-                        }
-                        let content: InputContent = Self::path_content(&path).await?;
-                        yield content;
-                    }
-                    Err(e) => eprintln!("{e:?}"),
                 }
+                InputSource::FsPath(ref path) => {
+                    if path.is_dir() {
+                        for entry in Input::walk_entries(
+                            path,
+                            file_extensions,
+                            skip_hidden,
+                            skip_gitignored,
+                        )? {
+                            let entry = entry?;
+                            if !Self::is_excluded_path(entry.path(), excluded_paths) {
+                                // Only yield files, not directories
+                                if entry.file_type().is_some_and(|ft| ft.is_file()) {
+                                    yield entry.path().to_string_lossy().to_string();
+                                }
+                            }
+                        }
+                    } else if !Self::is_excluded_path(path, excluded_paths) {
+                        yield path.to_string_lossy().to_string();
+                    }
+                }
+                InputSource::Stdin => yield "<stdin>".into(),
+                InputSource::String(_) => yield "<raw string>".into(),
             }
         }
     }
 
     /// Check if the given path was excluded from link checking
-    fn is_excluded_path(&self, path: &PathBuf) -> bool {
-        let Some(excluded_paths) = &self.excluded_paths else {
-            return false;
-        };
-        is_excluded_path(excluded_paths, path)
+    fn is_excluded_path(path: &Path, excluded_paths: &PathExcludes) -> bool {
+        excluded_paths.is_match(&path.to_string_lossy())
     }
 
     /// Get the input content of a given path
@@ -410,22 +541,40 @@ impl Input {
     }
 }
 
-/// Function for path exclusion tests
-///
-/// This is a standalone function to allow for easier testing
-fn is_excluded_path(excluded_paths: &[PathBuf], path: &PathBuf) -> bool {
-    for excluded in excluded_paths {
-        if let Ok(true) = utils::path::contains(excluded, path) {
-            return true;
-        }
+impl TryFrom<&str> for Input {
+    type Error = crate::ErrorKind;
+
+    fn try_from(value: &str) -> std::result::Result<Self, Self::Error> {
+        Self::from_value(value)
     }
-    false
+}
+
+/// Helper function to check if a file path matches any of the given extensions
+fn file_extensions_match(path: &Path, extensions: &FileExtensions) -> bool {
+    // If the path has no extension, check if we accept plaintext files
+    // NOTE: We treat files without extensions as plaintext, which might be problematic
+    // and is therefore subject to change
+    if path.extension().is_none() {
+        return extensions.contains("txt") || extensions.contains("");
+    }
+
+    // Otherwise, check if the extension is in our allowed list
+    path.extension()
+        .and_then(|ext| ext.to_str())
+        .is_some_and(|ext| extensions.contains(ext.to_lowercase()))
 }
 
 #[cfg(test)]
 mod tests {
     use super::*;
 
+    /// Function for path exclusion tests
+    ///
+    /// This is a standalone function to allow for easier testing
+    pub fn is_excluded_path(excluded_paths: &PathExcludes, path: &Path) -> bool {
+        excluded_paths.is_match(&path.to_string_lossy())
+    }
+
     #[test]
     fn test_input_handles_real_relative_paths() {
         let test_file = "./Cargo.toml";
@@ -434,14 +583,13 @@ mod tests {
         assert!(path.exists());
         assert!(path.is_relative());
 
-        let input = Input::new(test_file, None, false, None);
+        let input = Input::new(test_file, None, false);
         assert!(input.is_ok());
         assert!(matches!(
             input,
             Ok(Input {
                 source: InputSource::FsPath(PathBuf { .. }),
                 file_type_hint: None,
-                excluded_paths: None
             })
         ));
     }
@@ -454,33 +602,23 @@ mod tests {
         assert!(!path.exists());
         assert!(path.is_relative());
 
-        let input = Input::new(test_file, None, false, None);
+        let input = Input::from_value(test_file);
         assert!(input.is_err());
         assert!(matches!(input, Err(ErrorKind::InvalidFile(PathBuf { .. }))));
     }
 
-    #[test]
-    fn test_valid_extension() {
-        assert!(valid_extension(Path::new("file.md")));
-        assert!(valid_extension(Path::new("file.markdown")));
-        assert!(valid_extension(Path::new("file.html")));
-        assert!(valid_extension(Path::new("file.htm")));
-        assert!(valid_extension(Path::new("file.HTM")));
-        assert!(!valid_extension(Path::new("file.txt")));
-        assert!(!valid_extension(Path::new("file")));
-    }
-
     #[test]
     fn test_no_exclusions() {
         let dir = tempfile::tempdir().unwrap();
-        assert!(!is_excluded_path(&[], &dir.path().to_path_buf()));
+        assert!(!is_excluded_path(&PathExcludes::empty(), dir.path()));
     }
 
     #[test]
     fn test_excluded() {
         let dir = tempfile::tempdir().unwrap();
-        let path = dir.path().to_path_buf();
-        assert!(is_excluded_path(&[path.clone()], &path));
+        let path = dir.path();
+        let excludes = PathExcludes::new([path.to_string_lossy()]).unwrap();
+        assert!(is_excluded_path(&excludes, path));
     }
 
     #[test]
@@ -489,15 +627,14 @@ mod tests {
         let parent = parent_dir.path();
         let child_dir = tempfile::tempdir_in(parent).unwrap();
         let child = child_dir.path();
-        assert!(is_excluded_path(
-            &[parent.to_path_buf()],
-            &child.to_path_buf()
-        ));
+
+        let excludes = PathExcludes::new([parent.to_string_lossy()]).unwrap();
+        assert!(is_excluded_path(&excludes, child));
     }
 
     #[test]
     fn test_url_without_scheme() {
-        let input = Input::new("example.com", None, false, None);
+        let input = Input::from_value("example.com");
         assert_eq!(
             input.unwrap().source.to_string(),
             String::from("http://example.com/")
@@ -508,7 +645,7 @@ mod tests {
     #[cfg(windows)]
     #[test]
     fn test_windows_style_filepath_not_existing() {
-        let input = Input::new("C:\\example\\project\\here", None, false, None);
+        let input = Input::from_value("C:\\example\\project\\here");
         assert!(input.is_err());
         let input = input.unwrap_err();
 
@@ -528,7 +665,7 @@ mod tests {
         let dir = temp_dir();
         let file = NamedTempFile::new_in(dir).unwrap();
         let path = file.path();
-        let input = Input::new(path.to_str().unwrap(), None, false, None).unwrap();
+        let input = Input::from_value(path.to_str().unwrap()).unwrap();
 
         match input.source {
             InputSource::FsPath(_) => (),
@@ -540,33 +677,28 @@ mod tests {
     fn test_url_scheme_check_succeeding() {
         // Valid http and https URLs
         assert!(matches!(
-            Input::new("http://example.com", None, false, None),
+            Input::from_value("http://example.com"),
             Ok(Input {
                 source: InputSource::RemoteUrl(_),
                 ..
             })
         ));
         assert!(matches!(
-            Input::new("https://example.com", None, false, None),
+            Input::from_value("https://example.com"),
             Ok(Input {
                 source: InputSource::RemoteUrl(_),
                 ..
             })
         ));
         assert!(matches!(
-            Input::new(
-                "http://subdomain.example.com/path?query=value",
-                None,
-                false,
-                None
-            ),
+            Input::from_value("http://subdomain.example.com/path?query=value",),
             Ok(Input {
                 source: InputSource::RemoteUrl(_),
                 ..
             })
         ));
         assert!(matches!(
-            Input::new("https://example.com:8080", None, false, None),
+            Input::from_value("https://example.com:8080"),
             Ok(Input {
                 source: InputSource::RemoteUrl(_),
                 ..
@@ -578,19 +710,19 @@ mod tests {
     fn test_url_scheme_check_failing() {
         // Invalid schemes
         assert!(matches!(
-            Input::new("ftp://example.com", None, false, None),
+            Input::from_value("ftp://example.com"),
             Err(ErrorKind::InvalidFile(_))
         ));
         assert!(matches!(
-            Input::new("httpx://example.com", None, false, None),
+            Input::from_value("httpx://example.com"),
             Err(ErrorKind::InvalidFile(_))
         ));
         assert!(matches!(
-            Input::new("file:///path/to/file", None, false, None),
+            Input::from_value("file:///path/to/file"),
             Err(ErrorKind::InvalidFile(_))
         ));
         assert!(matches!(
-            Input::new("mailto:user@example.com", None, false, None),
+            Input::from_value("mailto:user@example.com"),
             Err(ErrorKind::InvalidFile(_))
         ));
     }
@@ -599,11 +731,11 @@ mod tests {
     fn test_non_url_inputs() {
         // Non-URL inputs
         assert!(matches!(
-            Input::new("./local/path", None, false, None),
+            Input::from_value("./local/path"),
             Err(ErrorKind::InvalidFile(_))
         ));
         assert!(matches!(
-            Input::new("*.md", None, false, None),
+            Input::from_value("*.md"),
             Ok(Input {
                 source: InputSource::FsGlob { .. },
                 ..
@@ -611,7 +743,7 @@ mod tests {
         ));
         // Assuming the current directory exists
         assert!(matches!(
-            Input::new(".", None, false, None),
+            Input::from_value("."),
             Ok(Input {
                 source: InputSource::FsPath(_),
                 ..
